/************************************************************************
 ************************************************************************
 ****                                                                ****
 ****    lcd.c : LN^^CvLCD\֐                         ****
 ****                                                                ****
 ****----------------------------------------------------------------****
 ****                                                                ****
 ************************************************************************
 ************************************************************************/
#include	"include/generic.h"
#include	"include/h8_3048.h"
#include	"include/config.h"

/************************************************************************
 *    LCD 萔`                                                      *
 ************************************************************************/
#define LCDCMDADR	(0xe7000)
#define LCDRAMADR	(0xe7001)
#define	LCDCMD		(*(UCHAR *)(LCDCMDADR))
#define	LCDRAM		(*(UCHAR *)(LCDRAMADR))

#define	LCDCMDID	(0x00)
#define LCDRAMID        (0x01)

/************************************************************************
 *    ֐Eϐ`                                                    *
 ************************************************************************/
extern	void	asmWriteByte(ULONG, UCHAR);
extern	void    asmLoop(UINT);

int	writeLcd(UCHAR, UCHAR);
int	outToLcd();
void	lcdInit();
void	initVCgRam();
void	initHCgRam();
void	wrietLcdRAM(UCHAR, UCHAR);
void	lcdWait();
void	lcdOut(char *);
void	lcdRawOut(char *, int);
void	lcdBackspace();
void	lcdMoveto(int, int);
void	lcdCommand(int);


#define	MAXLCDBUF       (160)
extern UCHAR		lcd_stat;
extern int	llow, lcol, blow, bcol;	/* Low/Column (Backing Store) */
extern	UCHAR	lnum[8];
extern  UCHAR   abuf[MAXLCDBUF];
extern  UCHAR	dbuf[MAXLCDBUF];
extern  int	bni, bno;

void
lcdFlush()
{
    int		s, d;
    while(1) {
        s = asmReadByte((ULONG)(&bni));
        d = asmReadByte((ULONG)(&bno));
        if (s == d) break;
    }
}

int
writeLcd(UCHAR id, UCHAR data)
{
    abuf[bni] = id;
    dbuf[bni] = data;
    bni++;
    if (bni == MAXLCDBUF)    bni = 0;
    if (bni == bno) {
        bno++;
        if (bno == MAXLCDBUF)  bno = 0;
    }
    return(0);
}

int
outToLcd()
{
    ULONG    adr;
    UCHAR    data;
    
    if (bni == bno)    return(0);		/* Mobt@`FbN */
    data = asmReadByte(LCDCMDADR) & 0x80;	/* LCD BUSY `FbN */
    if (data)          return(0);

    if (abuf[bno] == LCDCMDID)	adr = (ULONG)LCDCMDADR;
    else			adr = (ULONG)LCDRAMADR;
    /* adr  = abuf[bno]; */
    asmWriteByte(adr, dbuf[bno]);
    /* *adr = dbuf[bno]; */

    bno++;
    if (bno == MAXLCDBUF)  bno = 0;

    return(1);
}

/************************************************************************
 *    LCD CjVCY                                                *
 ************************************************************************/
void
lcdInit()
{
    int		i, n;
    UCHAR	command[10];
    
    /**** LCD Zbg ****/
    asmWriteByte(LCDCMDADR, 0x30);
    asmLoop(0x4000);
    asmWriteByte(LCDCMDADR, 0x30);
    asmLoop(800);

    /**** LCD ****/
    command[0] = 0x30;    command[1] = 0x3c;    command[2] = 0x08;
    command[3] = 0x01;    command[4] = 0x14;    command[5] = 0x06;
    command[6] = 0x0c;    command[7] = 0x02;    command[8] = 0x80;
    for(i=0; i<9; i++) {
	lcdWait();
	asmWriteByte(LCDCMDADR, command[i]);
    }

    /**** {ϐ̏****/
    bni = 0;
    bno = 0;
    llow = 0;
    lcol = 0;
    lcd_stat = 0x0c;
    lnum[0] = 0x00;
    lnum[1] = 0x40;
    lnum[2] = 0x14;
    lnum[3] = 0x54;
}

/************************************************************************
 *    CGRAMւ̏                                                 *
 ************************************************************************/
void
wrietLcdRAM(UCHAR adr, UCHAR ptn)
{
    lcdWait();
    asmWriteByte(LCDCMDADR, adr);
    lcdWait();
    asmWriteByte(LCDRAMADR, ptn);
}

/************************************************************************
 *    OtBbNptHg                                  *
 ************************************************************************/
/**** CG-RAMOttHg(c`)̏ ****/
char vfontList[36] = {
    (0x40 | (0x00 << 3) | 0x07),				/* 1 */
    (0x40 | (0x01 << 3) | 0x07), (0x40 | (0x01 << 3) | 0x06),	/* 2 */
    (0x40 | (0x02 << 3) | 0x07), (0x40 | (0x02 << 3) | 0x06),	/* 3 */
    (0x40 | (0x02 << 3) | 0x05),
    (0x40 | (0x03 << 3) | 0x07), (0x40 | (0x03 << 3) | 0x06),	/* 4 */
    (0x40 | (0x03 << 3) | 0x05), (0x40 | (0x03 << 3) | 0x04),
    (0x40 | (0x04 << 3) | 0x07), (0x40 | (0x04 << 3) | 0x06),	/* 5 */
    (0x40 | (0x04 << 3) | 0x05), (0x40 | (0x04 << 3) | 0x04),
    (0x40 | (0x04 << 3) | 0x03),
    (0x40 | (0x05 << 3) | 0x07), (0x40 | (0x05 << 3) | 0x06),	/* 6 */
    (0x40 | (0x05 << 3) | 0x05), (0x40 | (0x05 << 3) | 0x04),
    (0x40 | (0x05 << 3) | 0x03), (0x40 | (0x05 << 3) | 0x02),
    (0x40 | (0x06 << 3) | 0x07), (0x40 | (0x06 << 3) | 0x06),	/* 7 */
    (0x40 | (0x06 << 3) | 0x05), (0x40 | (0x06 << 3) | 0x04),
    (0x40 | (0x06 << 3) | 0x03), (0x40 | (0x06 << 3) | 0x02),
    (0x40 | (0x06 << 3) | 0x01),
    (0x40 | (0x07 << 3) | 0x07), (0x40 | (0x07 << 3) | 0x06),	/* 8 */
    (0x40 | (0x07 << 3) | 0x05), (0x40 | (0x07 << 3) | 0x04),
    (0x40 | (0x07 << 3) | 0x03), (0x40 | (0x07 << 3) | 0x02),
    (0x40 | (0x07 << 3) | 0x01), (0x40 | (0x07 << 3) | 0x00)
};

void
initVCgRam()
{
    int		i;
    
    /**** CGRAM f[^NA ****/
    for(i=0; i<0x40; i++) {
	wrietLcdRAM(0x40+(UCHAR)i, 0);
    }

    /**** c`o[OttHg ****/
    for(i=0; i<36; i++) {
        wrietLcdRAM(vfontList[i], 0xff);
    }
    
    asmWriteByte(LCDCMDADR, 0x80);
}

/**** CG-RAMOttHg(`)̏ ****/
void
initHCgRam()
{
    int		i;

    lcdWait();
    for(i=0; i<8; i++) wrietLcdRAM(0x40+(UCHAR)i, 0xf0);
    for(i=0; i<8; i++) wrietLcdRAM(0x48+(UCHAR)i, 0xf8);
    for(i=0; i<8; i++) wrietLcdRAM(0x50+(UCHAR)i, 0xfc);
    for(i=0; i<8; i++) wrietLcdRAM(0x58+(UCHAR)i, 0xfe);
    for(i=0; i<8; i++) wrietLcdRAM(0x60+(UCHAR)i, 0xff);
}

/************************************************************************
 *    LCD busy tbÕ`FbN                                       *
 ************************************************************************/
void
lcdWait()
{
    while(asmReadByte(LCDCMDADR) & 0x80) {
        ;
    }
}

/************************************************************************
 *    LCD LN^o                                                *
 ************************************************************************/
/**** o ****/
void
lcdOut(char *string)
{
    int		i=0;

    /* lcdWait(); */
    while(string[i]) {
	if (string[i] == '\n') {
	    if (llow == 3)	break;
	    else		llow++;
	    lcdMoveto(llow, 0);
	} else {
	    /* lcdWait(); */
            writeLcd((UCHAR)LCDRAMID, string[i]);
	    /* asmWriteByte(LCDRAMADR, string[i]); */
	    lcol++;
	}
	i++;
    }
}

/**** ړĕo ****/
void
lcdMoveOut(int line, int lcol, char *string)
{
    UCHAR	add;
    int		i;

    add = ((UCHAR)lcol + lnum[line]) | 0x80;
    /* lcdWait(); */
    /* LCDCMD = add; */
    writeLcd((UCHAR)LCDCMDID, add);
    llow = line;
    lcol = lcol;
    i = 0;
    while(string[i]) {
	/* lcdWait(); */
	if (string[i] == '\n') {
	    if (llow == 3)	break;
	    else		llow++;
	    lcdMoveto(llow, 0);
	} else {
	    /* asmWriteByte(LCDRAMADR, string[i]); */
            writeLcd((UCHAR)LCDRAMID, string[i]);
	    lcol++;
	}
	i++;
    }
}

void
lcdMoveOutn(int line, int lcol, char *string, int num)
{
    UCHAR	add;
    int		i;

    add = ((UCHAR)lcol + lnum[line]) | 0x80;
    /* lcdWait(); */
    /* LCDCMD = add; */
    writeLcd((UCHAR)LCDCMDID, add);
    llow = line;
    lcol = lcol;
    for(i=0; i<num; i++) {
	if (string[i] == '\n') {
	    if (llow == 3)	break;
	    else		llow++;
	    lcdMoveto(llow, 0);
	} else {
            writeLcd((UCHAR)LCDRAMID, string[i]);
	    lcol++;
	}
    }
}

/**** o(̂܂܏o) IWitHgp ****/
void
lcdRawOut(char *s, int n)
{
    int		i=0;

    for(i=0; i<n; i++) {
	/* lcdWait(); */
	/* asmWriteByte(LCDRAMADR, s[i]); */
        writeLcd((UCHAR)LCDRAMID, s[i]);
	lcol++;
    }
}

/**** LN^o ****/
void
lcdCharOut(char c)
{
    /* lcdWait(); */
    /* asmWriteByte(LCDRAMADR, c); */
    writeLcd((UCHAR)LCDRAMID, c);
    lcol++;
}
    
/**** obNXy[X ****/
void
lcdBackspace()
{
    lcdCommand(CUR_LEFT);
    lcdOut(" ");
    lcdCommand(CUR_LEFT);
}

/************************************************************************
 *    J[\ړ                                                      *
 ************************************************************************/
void
lcdMoveto(int line, int lcol)
{
    UCHAR	add;
    
    switch(line) {
    case 0:	add  = (UCHAR)lcol;		break;
    case 1:	add  = (UCHAR)lcol + 0x40;	break;
    case 2:	add  = (UCHAR)lcol + 0x14;	break;
    case 3:	add  = (UCHAR)lcol + 0x54;	break;
    default:	add  = (UCHAR)lcol;		break;
    }
    /* lcdWait(); */
    /* LCDCMD = add | 0x80; */
    writeLcd((UCHAR)LCDCMDID, (add | 0x80));
    llow = line;
    lcol = lcol;
}

/************************************************************************
 *    LCD R}ho                                                  *
 ************************************************************************/
/*
void
lcdCommand(int command)
{
    lcdWait();
    switch(command) {
    case CLEAR:		LCDCMD = 0x01;				break;
    case CUR_HOME:	LCDCMD = 0x02;				break;
    case LCD_ON:	LCDCMD = lcd_stat = lcd_stat | 0x0c;	break;
    case LCD_OFF:	LCDCMD = lcd_stat = lcd_stat & 0xfb;	break;
    case CUR_ON:	LCDCMD = lcd_stat = lcd_stat | 0x0a;	break;
    case CUR_OFF:	LCDCMD = lcd_stat = lcd_stat & 0xfd;	break;
    case BRINK_ON:	LCDCMD = lcd_stat = lcd_stat | 0x01;	break;
    case BRINK_OFF:	LCDCMD = lcd_stat = lcd_stat & 0xfe;	break;
    case CUR_LEFT:	LCDCMD = 0x10;				break;
    case CUR_RIGHT:	LCDCMD = 0x14;				break;
    case DISP_LEFT:	LCDCMD = 0x18;				break;
    case DISP_RIGHT:	LCDCMD = 0x1c;				break;
    }
}
*/

void
lcdCommand(int command)
{
    /* lcdWait(); */
    switch(command) {
    case CLEAR:
        writeLcd((UCHAR)LCDCMDID, 0x01);
        break;
    case CUR_HOME:
	writeLcd((UCHAR)LCDCMDID, 0x02);
        break;
    case LCD_ON:
	lcd_stat = lcd_stat | 0x0c;
        writeLcd((UCHAR)LCDCMDID, lcd_stat);
	break;
    case LCD_OFF:
	lcd_stat = lcd_stat & 0xfb;
        writeLcd((UCHAR)LCDCMDID, lcd_stat);
	break;
    case CUR_ON:
	lcd_stat = lcd_stat | 0x0a;
        writeLcd((UCHAR)LCDCMDID, lcd_stat);
	break;
    case CUR_OFF:
	lcd_stat = lcd_stat & 0xfd;
        writeLcd((UCHAR)LCDCMDID, lcd_stat);
	break;
    case BRINK_ON:
	lcd_stat = lcd_stat | 0x01;
        writeLcd((UCHAR)LCDCMDID, lcd_stat);
	break;
    case BRINK_OFF:
	lcd_stat = lcd_stat & 0xfe;
        writeLcd((UCHAR)LCDCMDID, lcd_stat);
	break;
    case CUR_LEFT:
        writeLcd((UCHAR)LCDCMDID, 0x10);
        break;
    case CUR_RIGHT:
        writeLcd((UCHAR)LCDCMDID, 0x14);
        break;
    case DISP_LEFT:
        writeLcd((UCHAR)LCDCMDID, 0x18);
        break;
    case DISP_RIGHT:
        writeLcd((UCHAR)LCDCMDID, 0x1c);
        break;
    }
}

/************************************************************************
 *    Biosp LCDo                                                    *
 ************************************************************************/
/**** o ****/
void
biosLcdOut(char *string)
{
    int		i=0;

    lcdWait();
    while(string[i]) {
	if (string[i] == '\n') {
	    if (llow == 3)	break;
	    else		llow++;
	    lcdMoveto(llow, 0);
	} else {
	    lcdWait();
	    asmWriteByte(LCDRAMADR, string[i]);
	    lcol++;
	}
	i++;
    }
}

void
biosLcdMove(int line, int lcol)
{
    UCHAR	add;
    
    switch(line) {
    case 0:	add  = (UCHAR)lcol;		break;
    case 1:	add  = (UCHAR)lcol + 0x40;	break;
    case 2:	add  = (UCHAR)lcol + 0x14;	break;
    case 3:	add  = (UCHAR)lcol + 0x54;	break;
    default:	add  = (UCHAR)lcol;		break;
    }
    lcdWait();
    LCDCMD = add | 0x80;
    llow = line;
    lcol = lcol;
}

/* end of file */

