/************************************************************************
 ************************************************************************
 ****                                                                ****
 ****    switch.c : XCb`͏                                 ****
 ****                                                                ****
 ************************************************************************
 ************************************************************************/
#include	"include/generic.h"
#include	"include/config.h"
#include	"include/h8_3048.h"

#define	PUSH_SW		(*(UCHAR *)(0xe4000))
#define TACT_SW		(*(UCHAR *)(0xe4001))

#define	RIGHTON		(0x0001)
#define LEFTON		(0x0002)
#define UPON		(0x0004)
#define DOWNON		(0x0008)

#define MAINDISP        (0x1000)
#define CHDISP		(0x1010)

#define	ENTER		(0x2000)
#define EDIT		(0x2010)
#define PGMINFO		(0x2020)

int	getAuxSendSw(ChAttr *);

/************************************************************************
 *    sw                                                            *
 *                                                                      *
 *    selsw  : LEDSW ォ0-3                                  *
 *    tactsw : ^NgXCb`@EォPs 0-3 2s 4-7         *
 *    mainsw : `l؂ւ^NgXCb`                         *
 ************************************************************************/
extern	UCHAR	pushprev, tactprev;
extern	UCHAR	selsw, tactsw, mainsw;
extern	UCHAR	pselsw, ptactsw, pmainsw;
extern	UCHAR	upselsw, uptactsw, upmainsw;
extern	UCHAR	dnselsw, dntactsw, dnmainsw;
extern	UCHAR	cmainsw;
extern	UINT	swcnt;
extern	int	sndsw;

void
swinit()
{
    pushprev = 0;
    tactprev = 0;
    selsw    = 0;    tactsw   = 0;    mainsw   = 0;
    pselsw   = 0;    ptactsw  = 0;    pmainsw  = 0;
    upselsw  = 0;    uptactsw = 0;    upmainsw = 0;
    dnselsw  = 0;    dntactsw = 0;    dnmainsw = 0;

    swcnt = 0;
    sndsw = 0;
}
    
void
swinput()
{
    UCHAR	now, stat;
    
    /**** Ol̕ۑ ****/
    ptactsw = tactsw;
    pselsw  = selsw;
    pmainsw = mainsw;

    /**** TACT SW ͏ ****/
    now = ~TACT_SW;
    stat = now & tactprev;
    tactsw = 0x00;
    if (stat & 0x80) tactsw = tactsw | 0x01;
    else	     tactsw = tactsw & 0xfe;
    if (stat & 0x40) tactsw = tactsw | 0x02;
    else	     tactsw = tactsw & 0xfd;
    if (stat & 0x20) tactsw = tactsw | 0x04;
    else	     tactsw = tactsw & 0xfb;
    if (stat & 0x10) tactsw = tactsw | 0x08;
    else	     tactsw = tactsw & 0xf7;
    if (stat & 0x08) tactsw = tactsw | 0x10;
    else	     tactsw = tactsw & 0xef;
    if (stat & 0x04) tactsw = tactsw | 0x20;
    else	     tactsw = tactsw & 0xdf;
    if (stat & 0x01) tactsw = tactsw | 0x40;
    else	     tactsw = tactsw & 0xbf;
    if (stat & 0x02) tactsw = tactsw | 0x80;
    else	     tactsw = tactsw & 0x7f;
    tactprev = now;

    /**** main SW  ****/
    now = ~PUSH_SW;
    stat = now & pushprev;
    mainsw = 0;
    if (stat & 0x80) mainsw = 1;
    else	     mainsw = 0;

    /**** AUX Send Select SW ͏ ****/
    if (sndsw == ON) {
        selsw = 0;
        if (stat & 0x04) selsw = selsw | 0x01;
        else	         selsw = selsw & 0xfe;
        if (stat & 0x01) selsw = selsw | 0x02;
        else	         selsw = selsw & 0xfd;
        if (stat & 0x08) selsw = selsw | 0x04;
        else	         selsw = selsw & 0xfb;
        if (stat & 0x02) selsw = selsw | 0x08;
        else	         selsw = selsw & 0xf7;
    }
    pushprev = now;

    /**** XCb`Ԑݒ ****/
    /* オGbW */
    uptactsw =  ptactsw & ~tactsw;
    upmainsw = ~pmainsw &  mainsw;
    upselsw  = ~pselsw  &  selsw;

    /* GbW */
    dntactsw = ~ptactsw &  tactsw;
    dnmainsw =  pmainsw & ~mainsw;
    dnselsw  =  pselsw  & ~selsw;

    /**** ςȂ(ch؂ւpCXCb`̂ ****/
    if (mainsw == 1) {
	swcnt++;
	if (swcnt >= 40) {
	    cmainsw = 1;
	    upmainsw = 0;
	    dnmainsw = 0;
	}
    } else {
	if (swcnt >= 40) {
	    upmainsw = 0;
	    dnmainsw = 0;
	}
	swcnt = 0;
	cmainsw = 0;
    }
}

UCHAR
getSwStat(int sw, int stat)
{
    UCHAR	rtn;

    rtn = 0;
    if (sw == MAINSW) {
	switch(stat) {
	case DOWN_STAT:		rtn = dnmainsw;	break;
	case UP_STAT:		rtn = upmainsw;	break;
	case PUSH_STAT:		rtn = mainsw;	break;
	}
    } else if (sw == SELSW) {
	switch(stat) {
	case DOWN_STAT:		rtn = dnselsw;	break;
	case UP_STAT:		rtn = upselsw;	break;
	case PUSH_STAT:		rtn = selsw;	break;
	}
    } else if (sw == TACTSW) {
	switch(stat) {
	case DOWN_STAT:		rtn = dntactsw;	break;
	case UP_STAT:		rtn = uptactsw;	break;
	case PUSH_STAT:		rtn = tactsw;	break;
	}
    }
    return(rtn);
}

int
checksw(int mask)
{
    int		rtn;
    
    switch(mask) {
    case MAINSW:	rtn = dnmainsw;		break;
    case CMAINSW:	rtn = cmainsw;		break;
    case SELSW0:	rtn = upselsw & 0x01;	break;
    case SELSW1:	rtn = upselsw & 0x02;	break;
    case SELSW2:	rtn = upselsw & 0x04;	break;
    case SELSW3:	rtn = upselsw & 0x08;	break;
    case TACTSW0:	rtn = dntactsw & 0x01;	break;
    case TACTSW1:	rtn = dntactsw & 0x02;	break;
    case TACTSW2:	rtn = dntactsw & 0x04;	break;
    case TACTSW3:	rtn = dntactsw & 0x08;	break;
    case TACTSW4:	rtn = dntactsw & 0x10;	break;
    case TACTSW5:	rtn = dntactsw & 0x20;	break;
    case TACTSW6:	rtn = dntactsw & 0x40;	break;
    case TACTSW7:	rtn = dntactsw & 0x80;	break;
    }
    return(rtn);
}

int
auxswUpdateSet(int onoff)
{
    if(onoff == ON) {
        sndsw = 1;
    } else {
        sndsw = 0;
    }
}

int
getAuxSendSw(ChAttr *chd)
{
    UCHAR	*sw, *flg;

    if (sndsw == 0)    return 0;
    
    sw  = &(chd->sndsw);
    flg = &(chd->sflg);

    if(upselsw & 0x01) {
        if (*sw & 0x01) *sw &= 0xfe;
        else            *sw |= 0x01;
        *flg |= 0x11;
    }
    if (upselsw & 0x02) {
        if (*sw & 0x02) *sw &= 0xfd;
        else            *sw |= 0x02;
        *flg |= 0x22;
    }
    if(upselsw & 0x04) {
        if (*sw & 0x04) *sw &= 0xfb;
        else            *sw |= 0x04;
        *flg |= 0x44;
    }
    if (upselsw & 0x08) {
        if (*sw & 0x08) *sw &= 0xf7;
        else            *sw |= 0x08;
        *flg |= 0x88;
    }
    return 1;
}
    
/* end of file */
